// Transport equation and Conservation Law
// Second order methods
//// Space discretization
L=5;
dx=0.01;
space=(0:dx:L)';
//// Time discretization
T=1;
dt=dx*0.95;
///// space should be a column vector
//// Velocity of the transport equation -- a=1 
deff('[y]=a(x)','y=1'); 
//// Initial datum 2
  space1=space(space<1);
  space2=space((space>=1)&(space<=2));
  space3=space(space>2);
  uinit=[zeros(space1);ones(space2); zeros(space3)];
///// FLux limiter 1 : Minmod
deff('[y]=Minmod(x)','y=max(zeros(x),min(x,ones(x)))');
///// FLux limiter 2 : Roe's superbee
deff('[y]=Roe(x)','y=max(zeros(x),max(min(x,2*ones(x)),min(ones(x),2*x)))');
///// FLux limiter 3 : Van Leer
deff('[y]=VanLeer(x)','y=(x+abs(x))./(ones(x)+abs(x))');
//// Comparison of the different flux limiter functions
uMm=FluxLimiter(T,dt,L,dx,uinit,a,Minmod);
plot(space,uMm);
uRoe=FluxLimiter(T,dt,L,dx,uinit,a,Roe);
plot(space,uRoe,'r');
uVL=FluxLimiter(T,dt,L,dx,uinit,a,VanLeer);
plot(space,uVL,'g');
legend('Minmod','Roe','Van Leer');
///////////////////// Computation of the error
L=5;
SpaceStep=[0.1, 0.05, 0.01, 0.005, 0.001, 0.0005];
for k=1:length(SpaceStep),
dx=SpaceStep(k);
space=(0:dx:L)';
//// Time discretization
T=1;
dt=0.95*dx;
time=0:dt:T;
    Nt=length(time);
Tsimu=dt*Nt;
//// Velocity of the transport equation -- a=1 
deff('[y]=a(x)','y=1');
//// Initial datum 2
  space1=space(space<1);
  space2=space((space>=1)&(space<=2));
  space3=space(space>2);
  uinit=[zeros(space1);ones(space2); zeros(space3)];
//// Exact solution  2
     space1b=space(space<1+Tsimu);
     space2b=space((space>=1+Tsimu)&(space<=2+Tsimu));
     space3b=space(space>2+Tsimu);
     uexact=[zeros(space1b);ones(space2b); zeros(space3b)];
//// Approximated solution
uMm=FluxLimiter(T,dt,L,dx,uinit,a,Minmod);
ErrorMm(k)=dx*norm(uMm-uexact,1);
end
//// Graph of the errors
figure(1);
plot2d(SpaceStep,ErrorMm,logflag="ll",style=1);
plot2d(SpaceStep, SpaceStep.^2,logflag="ll",style=2);
//// Legend for the graph
legend('Minmod','order 2');

